package com.franckbarbier.Heatwave;

import com.franckbarbier.Temperature.*;

public class Heatwave_period {

    Heatwave_period(java.time.ZonedDateTime peak, Temperature temperature) throws Exception {
        switch (Season(peak)) { // Which season?
            case Summer:
                if (temperature.greaterThan(Hot.Ceil)) {
                    if (temperature.greaterThan(Hotter.Ceil)) {
                        throw new Hottest();
                    } else {
                        throw new Hotter();
                    }
                }
                throw new Hot();
            case Autumn: // No heatwave:
                throw new Warm();
            case Winter:
                // ...
                break;
            case Spring: // June 18 France heatwave:
                if (temperature.greaterThan(Warm.Ceil)) {
                    if (temperature.greaterThan(Hot.Ceil)) {
                        if (temperature.greaterThan(Hotter.Ceil)) {
                            throw new Hottest();
                        } else {
                            throw new Hotter();
                        }
                    }
                }
                throw new Warm();
            case Unknown:
            default:
                throw new RuntimeException("Impossible to be there...");
        }
    }

    public enum Cities {

        Paris("Europe/Paris"), Sydney("Australia/Sydney");
        private final String _literal;

        Cities(String literal) {
            _literal = literal;
        }

        @Override
        public String toString() {
            return _literal;
        }
    }

    public enum Seasons {
        Summer, Autumn, Winter, Spring, Unknown
    }

    public static Seasons Season(java.time.ZonedDateTime date_time) {
        /*
        * On ne gère que Paris et Sydney : (!Paris => Sydney), ce qui est équivalent à (Paris || !Sydney)
         */
        assert (date_time.getZone().getId().equals(Cities.Paris) || !date_time.getZone().getId().equals(Cities.Sydney));

        int day = date_time.getDayOfMonth(); // [1 - 31]
        java.time.Month month = date_time.getMonth(); // Type énuméré...

        switch (month) {
            case JULY:
            case AUGUST:
                // C'est l'hiver à Sydney :
                return date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Summer : Seasons.Winter;
            case SEPTEMBER:
                return (day < 21)
                        ? date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Summer : Seasons.Winter
                        : date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Autumn : Seasons.Spring;
            case OCTOBER:
            case NOVEMBER:
                // C'est le printemps à Sydney :
                return date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Autumn : Seasons.Spring;
            case DECEMBER:
                return (day < 21)
                        ? date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Autumn : Seasons.Spring
                        : date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Winter : Seasons.Summer;
            case JANUARY:
            case FEBRUARY:
                // C'est l'été à Sydney :
                return date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Winter : Seasons.Summer;
            case MARCH:
                return (day < 21)
                        ? date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Winter : Seasons.Summer
                        : date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Spring : Seasons.Autumn;
            case APRIL:
            case MAY:
                // C'est l'automne à Sydney :
                return date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Spring : Seasons.Autumn;
            case JUNE:
                return (day < 21)
                        ? date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Spring : Seasons.Autumn
                        : date_time.getZone().getId().equals(Cities.Paris) ? Seasons.Summer : Seasons.Winter;
            default:
                return Seasons.Unknown;
        }
    }
}
